local msq, msqTools
msq = LibStub("Masque")
if msq then
	msqTools = msq:Group("FarmControls")
end

local SZ = LibStub("LibBabble-SubZone-3.0"):GetLookupTable()

local _
local ButtonSpacing = 6

local FarmControls = CreateFrame("frame","FarmControls",UIParent)
FarmControls:RegisterEvent("PLAYER_LOGIN")
FarmControls:RegisterEvent("PLAYER_ENTERING_WORLD")
local function FarmControls_EventHandler(self, event, ...)
	if event == "PLAYER_LOGIN" then
		self:Initialize()
	elseif event == "PLAYER_ENTERING_WORLD" then
		FarmControls:RegisterEvent("BAG_UPDATE")
		FarmControls:RegisterEvent("ZONE_CHANGED")
		FarmControls:UnregisterEvent("PLAYER_ENTERING_WORLD")
	elseif event == "ZONE_CHANGED" then
		FarmControls:UpdateLayout()
	elseif event == "BAG_UPDATE" then
		FarmControls:UpdateLayout()
	elseif event == "PLAYER_REGEN_ENABLED" then
		FarmControls:UnregisterEvent("PLAYER_REGEN_ENABLED")
		if self.Initialized then
			self:UpdateLayout()
		else
			self:Initialize()
		end	
	end
end
FarmControls:SetScript("OnEvent",FarmControls_EventHandler)
FarmControls:SetMovable(true)

FarmControls.tools = CreateFrame("Frame","FarmControlsTools",FarmControls);
FarmControls.tools:SetScale(1.5)
FarmControls.tools.items = {
	{id=89815}, --Master Plow
	{id=89880}, --Dented Shovel
	{id=80513}, --Vintage Bug Sprayer
	{id=79104}, --Rusty Watering Can
}

FarmControls.seeds = CreateFrame("Frame","FarmControlsSeeds",FarmControls);
FarmControls.seeds:SetScale(1)
FarmControls.seeds.items = {
	{id=79102}, --Green Cabbage Seeds
	{id=89328}, --Jade Squash Seeds
	{id=80590}, --Juicycrunch Carrot Seeds
	{id=80592}, --Mogu Pumpkin Seeds
	{id=80594}, --Pink Turnip Seeds
	{id=80593}, --Red Blossom Leek Seeds
	{id=80591}, --Scallion Seeds
	{id=89329}, --Striped Melon Seeds
	{id=80595}, --White Turnip Seeds
	{id=89326}, --Witchberry Seeds
	{id=85219}, --Ominous Seed
	{id=85216}, --Enigma Seed
	{id=85217}, --Magebulb Seed
	{id=89202}, --Raptorleaf Seed
	{id=85215}, --Snakeroot Seed
	{id=89233}, --Songbell Seed
	{id=89197}, --Windshear Cactus Seed
}

function FarmControls:Initialize()
	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end
	if FarmControlsData == nil then
		FarmControlsData = {
			X = 0,
			Y = -250,
		}
	end
	self:CreateButtons(FarmControls.tools)
	self:CreateButtons(FarmControls.seeds)
	self:UpdateLayout()
	self.Initialized = true
end

function FarmControls:CreateButtons(bar)
	for i,item in ipairs(bar.items) do

		local icon = select(10,GetItemInfo(item.id))

		item.btn = CreateFrame("Button", bar:GetName().."Button"..i, bar, "SecureActionButtonTemplate");
		item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")

		item.btn.count = item.btn:CreateFontString(item.btn:GetName().."Count","ARTWORK","NumberFontNormal")
		item.btn.count:SetJustifyH("RIGHT")
		item.btn.count:SetPoint("BottomRight",-2,2)

		item.btn.border = item.btn:CreateTexture(item.btn:GetName().."Border","OVERLAY")
		item.btn.border:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
		item.btn.border:SetBlendMode("ADD")
		item.btn.border:SetSize(62,62)
		item.btn.border:SetPoint("CENTER",0,0)
		item.btn.border:Hide()

		item.btn:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2")
		item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
		item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
		item.btn:SetSize(32,32)
		
		item.btn:RegisterForClicks("AnyUp")
		item.btn:SetAttribute("type", "item");
		item.btn:SetAttribute("item1", "item:"..item.id);
		item.btn.icon:SetTexture(icon)
		item.btn:SetScript("OnMouseDown",FarmControls.OnMouseDown)
		item.btn:SetScript("OnMouseUp",FarmControls.OnMouseUp)
		item.btn:SetScript("OnHide",FarmControls.OnHide)
		item.btn:SetScript("OnEnter",FarmControls.OnEnter)
		item.btn:SetScript("OnLeave",function() GameTooltip_Hide() end)
		if msqTools then
			msqTools:AddButton(item.btn)
		end
	end
end

function FarmControls:OnEnter()
	local _, link = GetItemInfo(self:GetAttribute("item1"))
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetHyperlink(link)
	GameTooltip:Show()
end

function FarmControls:OnMouseDown(button)
	if IsShiftKeyDown() and button == "LeftButton" and not FarmControls.isMoving then
		FarmControls:StartMoving()
		FarmControls.isMoving = true
	elseif IsShiftKeyDown() and button == "RightButton" and FarmControls.isMoving then
		FarmControls:StopMovingOrSizing();
		FarmControlsData.X, FarmControlsData.Y = 0, -250
		FarmControls:SetPoint("Center",UIParent,"Center",0,-250)
		FarmControls.isMoving = false;
	end
end

function FarmControls:OnMouseUp(button)
	if button == "LeftButton" and FarmControls.isMoving then
		FarmControls:StopMovingOrSizing()
		_,_,_,FarmControlsData.X,FarmControlsData.Y = FarmControls:GetPoint(1)
		FarmControls.isMoving = false
   end
end

function FarmControls:OnHide()
	if ( FarmControls.isMoving ) then
		FarmControls:StopMovingOrSizing();
		_,_,_,FarmControlsData.X,FarmControlsData.Y = FarmControls:GetPoint(1)
		FarmControls.isMoving = false;
	end
end

function FarmControls:UpdateBar(bar)
	local last
	local shown = 0
	for _,item in ipairs(bar.items) do
		local ItemCount = GetItemCount(item.id)
		if ItemCount > 0 then
			if shown % 8 == 0 then
				local row = math.floor(shown/8)
				item.btn:SetPoint("TopLeft",bar,"TopLeft", ButtonSpacing/2, -(ButtonSpacing * .5 ) - row*item.btn:GetHeight() - row*ButtonSpacing)
				last = item.btn
			else
				item.btn:SetPoint("TopLeft",last,"TopRight",ButtonSpacing,0)
				last = item.btn
			end
			if IsConsumableItem(item.id) then
				local count = _G[item.btn:GetName().."Count"]
				if ItemCount > 999 then
					count:SetText("*")
				else
					count:SetText(ItemCount)
				end
			end
			local bag, slot = self:FindItemInBags(item.id)
			if bag and slot then
				item.btn:SetAttribute("item2",bag.." "..slot)
			end
			item.btn:Show()
			shown = shown + 1
		else
			item.btn:Hide()
		end
	end
	
	if last ~= nil then
		bar:SetWidth(math.min(8,shown) * (last:GetWidth() + ButtonSpacing))
		bar:SetHeight(math.ceil(shown/8) * (last:GetHeight() + ButtonSpacing))
		bar:Show()
	else
		bar:Hide()
	end
end

function FarmControls:UpdateLayout()

	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end

	if GetSubZoneText() ~= "" and 
	   GetSubZoneText() ~= SZ["Sunsong Ranch"] and  
	   GetSubZoneText() ~= SZ["The Halfhill Market"] then
		FarmControls:Hide()
		if GetSubZoneText() ~= SZ["Halfhill"] then
			ClearCursor()
			for i,item in ipairs(FarmControls.tools.items) do
				local bag, slot = self:FindItemInBags(item.id)
				if bag and slot then
					PickupContainerItem(bag,slot)
					if CursorHasItem() then
						local _, id, link = GetCursorInfo()
						if id == item.id then
							print("Leaving "..SZ["Halfhill"]..". Dropping "..link..".")
							DeleteCursorItem()
						end
					end
				end
			end
		end
		return
	end

	FarmControls:UpdateBar(self.tools)
	FarmControls:UpdateBar(self.seeds)
		
	local th = (FarmControls.tools:IsShown() and FarmControls.tools:GetHeight()*FarmControls.tools:GetScale() or 0)
	local sh = (FarmControls.seeds:IsShown() and FarmControls.seeds:GetHeight()*FarmControls.seeds:GetScale() or 0)
	FarmControls:SetHeight(24 + th + sh)
	
	local tw = (FarmControls.tools:IsShown() and FarmControls.tools:GetWidth()*FarmControls.tools:GetScale() or 0)
	local sw = (FarmControls.seeds:IsShown() and FarmControls.seeds:GetWidth()*FarmControls.seeds:GetScale() or 0)
	FarmControls:SetWidth(24 + math.max(tw,sw))
	
	if FarmControls.tools:IsShown() or FarmControls.seeds:IsShown() then
		FarmControls:Show()
	else
		FarmControls:Hide()
	end

	FarmControls.seeds:SetPoint("Top",FarmControls,"Top",0,-15)
	FarmControls.tools:SetPoint("Bottom",FarmControls,"Bottom",0,7)
	FarmControls.tools:SetPoint("Bottom",FarmControls,"Bottom",0,7)
	FarmControls:SetPoint("Center",UIParent,"Center",FarmControlsData.X,FarmControlsData.Y)
end

function FarmControls:FindItemInBags(item)
	for container = 0, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(container)
		if numSlots then
			for slot = 0, numSlots do
				for slot = 1, numSlots do
					if item == GetContainerItemID(container, slot) then
						return container, slot
					end
				end
			end
		end
	end
end
